use nu_plugin::{EngineInterface, EvaluatedCall, SimplePluginCommand};
use nu_protocol::{record, Category, Example, LabeledError, Signature, Value};

use crate::OncePlugin;

pub struct Scenario;

impl SimplePluginCommand for Scenario {
    type Plugin = OncePlugin;

    fn name(&self) -> &str {
        "once scenario"
    }

    fn usage(&self) -> &str {
        "Manage once scenarios on local or remote servers."
    }

    fn extra_usage(&self) -> &str {
        r"This command shows all available scenarios. To manage these scenarios use the sub commands, e.g. 

  > once scenario.deploy"
    }

    fn signature(&self) -> Signature {
        Signature::build(self.name()).category(Category::Experimental)
    }

    fn search_terms(&self) -> Vec<&str> {
        vec!["once", "scenario"]
    }

    fn examples(&self) -> Vec<Example> {
        vec![Example {
            example: "once scenario",
            description: "List all scenarios",
            result: Some(Value::test_list(vec![Value::test_record(record! {
                "name" => Value::test_string("test"),
                "filename" => Value::test_string("test.scenario.env")
            })])),
        }]
    }

    fn run(
        &self,
        _plugin: &OncePlugin,
        _engine: &EngineInterface,
        call: &EvaluatedCall,
        _input: &Value,
    ) -> Result<Value, LabeledError> {
        let head = call.head;
        let list = vec![
            Value::record(
                record! {
                    "name" => Value::test_string("dev"),
                    "filename" => Value::test_string("dev.scenario.env")
                },
                head,
            ),
            Value::record(
                record! {
                    "name" => Value::test_string("certbot"),
                    "filename" => Value::test_string("certbot.scenario.env")
                },
                head,
            ),
        ];
        Ok(Value::list(list, head))
    }
}

#[test]
fn test_examples() -> Result<(), nu_protocol::ShellError> {
    use nu_plugin_test_support::PluginTest;
    PluginTest::new("once", OncePlugin.into())?.test_command_examples(&Scenario)
}
